"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resolveNode = exports.AddQueriesToSubscriptionsPlugin = exports.MutationPayloadQueryPlugin = exports.ClientMutationIdDescriptionPlugin = exports.SubscriptionPlugin = exports.MutationPlugin = exports.QueryPlugin = exports.NodePlugin = exports.StandardTypesPlugin = exports.SwallowErrorsPlugin = exports.defaultPlugins = exports.buildSchema = exports.getBuilder = exports.LiveCoordinator = exports.LiveMonitor = exports.LiveProvider = exports.LiveSource = exports.singularize = exports.pluralize = exports.upperCamelCase = exports.constantCase = exports.camelCase = exports.upperFirst = exports.formatInsideUnderscores = exports.constantCaseAll = undefined;

var _utils = require("./utils");

Object.defineProperty(exports, "constantCaseAll", {
  enumerable: true,
  get: function () {
    return _utils.constantCaseAll;
  }
});
Object.defineProperty(exports, "formatInsideUnderscores", {
  enumerable: true,
  get: function () {
    return _utils.formatInsideUnderscores;
  }
});
Object.defineProperty(exports, "upperFirst", {
  enumerable: true,
  get: function () {
    return _utils.upperFirst;
  }
});
Object.defineProperty(exports, "camelCase", {
  enumerable: true,
  get: function () {
    return _utils.camelCase;
  }
});
Object.defineProperty(exports, "constantCase", {
  enumerable: true,
  get: function () {
    return _utils.constantCase;
  }
});
Object.defineProperty(exports, "upperCamelCase", {
  enumerable: true,
  get: function () {
    return _utils.upperCamelCase;
  }
});
Object.defineProperty(exports, "pluralize", {
  enumerable: true,
  get: function () {
    return _utils.pluralize;
  }
});
Object.defineProperty(exports, "singularize", {
  enumerable: true,
  get: function () {
    return _utils.singularize;
  }
});

var _Live = require("./Live");

Object.defineProperty(exports, "LiveSource", {
  enumerable: true,
  get: function () {
    return _Live.LiveSource;
  }
});
Object.defineProperty(exports, "LiveProvider", {
  enumerable: true,
  get: function () {
    return _Live.LiveProvider;
  }
});
Object.defineProperty(exports, "LiveMonitor", {
  enumerable: true,
  get: function () {
    return _Live.LiveMonitor;
  }
});
Object.defineProperty(exports, "LiveCoordinator", {
  enumerable: true,
  get: function () {
    return _Live.LiveCoordinator;
  }
});

var _util = require("util");

var _util2 = _interopRequireDefault(_util);

var _SchemaBuilder = require("./SchemaBuilder");

var _SchemaBuilder2 = _interopRequireDefault(_SchemaBuilder);

var _plugins = require("./plugins");

var _resolveNode = require("./resolveNode");

var _resolveNode2 = _interopRequireDefault(_resolveNode);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const getBuilder = exports.getBuilder = async (plugins, options = {}) => {
  const builder = new _SchemaBuilder2.default(options);
  for (let i = 0, l = plugins.length; i < l; i++) {
    const plugin = plugins[i];
    if (typeof plugin !== "function") {
      throw new Error(`Expected a list of plugin functions, instead list contained a non-function at index ${i}: ${_util2.default.inspect(plugin)}`);
    }
    builder._setPluginName(plugin.displayName || plugin.name);
    await plugin(builder, options);
    builder._setPluginName(null);
  }
  return builder;
};

const buildSchema = exports.buildSchema = async (plugins, options = {}) => {
  const builder = await getBuilder(plugins, options);
  return builder.buildSchema();
};

const defaultPlugins = exports.defaultPlugins = [_plugins.SwallowErrorsPlugin, _plugins.StandardTypesPlugin, _plugins.NodePlugin, _plugins.QueryPlugin, _plugins.MutationPlugin, _plugins.SubscriptionPlugin, _plugins.ClientMutationIdDescriptionPlugin, _plugins.MutationPayloadQueryPlugin, _plugins.AddQueriesToSubscriptionsPlugin];

exports.SwallowErrorsPlugin = _plugins.SwallowErrorsPlugin;
exports.StandardTypesPlugin = _plugins.StandardTypesPlugin;
exports.NodePlugin = _plugins.NodePlugin;
exports.QueryPlugin = _plugins.QueryPlugin;
exports.MutationPlugin = _plugins.MutationPlugin;
exports.SubscriptionPlugin = _plugins.SubscriptionPlugin;
exports.ClientMutationIdDescriptionPlugin = _plugins.ClientMutationIdDescriptionPlugin;
exports.MutationPayloadQueryPlugin = _plugins.MutationPayloadQueryPlugin;
exports.AddQueriesToSubscriptionsPlugin = _plugins.AddQueriesToSubscriptionsPlugin;
exports.resolveNode = _resolveNode2.default;
//# sourceMappingURL=index.js.map